using System;
using System.Collections.Generic;
using System.Text;
using ceometric.ComputationalGeometry;
using ceometric.VectorGeometry;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// You must reference both V# and G# first to run this program
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

namespace ContourLines
{
    /// <summary>
    /// A simple program that
    /// 1. Generates some sample points
    /// 2. Triangulates the sample points
    /// 3. Computes contour lines of the triangulated surface by
    ///    intersecting each triangle of the surface with a slicing plane
    /// 4. Outputs the surface and the contour lines in dwf format.
    /// </summary>
    class Program
    {
        static void Main(string[] args)
        {
            //---------------------------------------------------------------------
            // Create a sample set of vertices. We will use the in-build sample point
            // generator to create a starfish-shaped structure with 10000 vertices:
            PointSet ps = SamplePoints.SupershapeStarfish(10000);

            //---------------------------------------------------------------------
            //Triangulate the vertices:
            DelaunayTriangulation2d dt = new DelaunayTriangulation2d();
            dt.TriangulationPoints = ps;
            dt.Triangulate();
            List<Triangle> triangles = dt.GetTriangles(); //Get the triangles of the surface

            //---------------------------------------------------------------------
            // To compute the contour lines at a height z = 0.10, we simply create
            // a plane parallel to the XY-plane at z=0.10 and intersect every
            // triangle of the surface with the slicing plane:
            Plane slicingPlane = new Plane(new Point(0, 0, 0.10), new Vector3d(0, 0, 1));
            List<Edge> contourEdges = new List<Edge>();
            foreach (Triangle t in triangles)
            {
                Edge contourEdge = t.Intersect3d(slicingPlane);
                if (contourEdge != null) contourEdges.Add(contourEdge);
            }

            //---------------------------------------------------------------------
            // Visualize in dxf format
            // Get the free DWG TrueView viewer at www.autodesk.com to view dxf
            // Create a scene and layers
            Scene sc = new Scene();
            Scene.Layer eLayer = new Scene.Layer("Contour_lines", 1);
            Scene.Layer tLayer = new Scene.Layer("Delaunay_Triangles", 7);

            // Add objects to the scene
            sc.Add(contourEdges, eLayer);
            sc.Add(triangles, tLayer);

            // Output the scene as dxf
            string outfile = @"..\..\contour_lines.dxf";
            sc.WriteDxf(outfile);
            Console.WriteLine("dxf written to " + outfile);

            Console.WriteLine("Hit any key to continue.");
            Console.ReadLine();
        }
    }
}